import java.util.Arrays;

public class javacbc_test {
	static final Double INFIN = Double.MAX_VALUE;
	static{System.loadLibrary("javacbc");}
	public static void main(String args[]){
		//TODO: If flag -f file.lp then load file.lp in solver instead of building it
		
		//If no arg, display on line help
		//TODO: Use jargs lib? http://jargs.sourceforge.net/


		//Test#1: test() from Cbc
		System.out.println("########### Test #1 ###########");
		System.out.println(javacbc.test());
		System.out.println("########### Test #1 ###########\n");

		//Test #2: solve continuous lp from file
		System.out.println("########### Test #2 ###########");
		SWIGTYPE_p_OsiClpSolverInterface solverCont = javacbc.solver_from_file("/usr/share/coin/Data/Sample/p0033.mps");
		SWIGTYPE_p_CbcModel modelCont = javacbc.makeCbcModel(solverCont);
		javacbc.setLogLevel(modelCont,0);
		javacbc.setObjSense(modelCont,1.0); //Minimization
		javacbc.initialSolve(modelCont);
		javacbc.branchAndBound(modelCont,0);
		showResult(modelCont);
		System.out.println("########### Test #2 ###########\n");

		//Test #3: solve Mip from file, using solve_model
		System.out.println("########### Test #3 ###########");
		SWIGTYPE_p_OsiClpSolverInterface solverMip1 = javacbc.solver_from_file("/usr/share/coin/Data/Sample/exmip1.mps");
		javacbc.solve_model(solverMip1, true, false, 5, -1); //presolve, no clique, 5 passes, no max time
		System.out.println("########### Test #3 ###########\n");

		//test #4: solve Mip from file, using CbcModel
		System.out.println("########### Test #4 ###########");
		SWIGTYPE_p_OsiClpSolverInterface solverMip2 = javacbc.solver_from_file("/usr/share/coin/Data/Sample/exmip1.mps");
		SWIGTYPE_p_CbcModel modelMip2 = javacbc.makeCbcModel(solverMip2);
		javacbc.setLogLevel(modelMip2,0);
		javacbc.setObjSense(modelMip2,1.0); //Maximization
		javacbc.initialSolve(modelMip2);
		javacbc.branchAndBound(modelMip2,0);
		showResult(modelMip2);
		System.out.println("########### Test #4 ###########\n");



		//Test #5: build a model in java
		System.out.println("########### Test #5 ###########");
		int k1=2;
		int k2=2;
		int numberRows=1+k1+k1*k2; //Number of constraints
		int numberColumns=8; //Nombre de vars: n
		int numberElements=numberColumns*numberRows; //Nombre on non zero in A
		int[] rowIndices = new int[numberElements];
		int[] colIndices = new int[numberElements]; 
		double[] elements  =new double[numberElements];
		//Make matrix A
		int t=0;
		for (int i=0;i<numberRows;i++){
			for (int j=0;j<numberColumns;j++){
				rowIndices[t]=i;
				colIndices[t]=j;
				elements[t]=i*j+1;
				t++;
			}
		}
		//Transform the data about A into a CoinPackedMatrix
		SWIGTYPE_p_CoinPackedMatrix matrix = javacbc.CPM(rowIndices, colIndices, elements, numberElements);

		//Other data
		double objective[]={1,2,3,4,5,6,7,8}; //Objective function
		double rowLower[]={-INFIN,-INFIN,-INFIN,-INFIN,-INFIN,-INFIN,-INFIN}; //LHS <= constraints
		double rowUpper[]={50,51,40,55,54,65,46}; //contraints <= RHS
		double colLower[]={0,0,0,0,0,0,0,0}; //lower bound <= x
		double colUpper[]={1,1,1,1,1,1,1,1}; //x <= upper bound

		//Instanciate a solver
		SWIGTYPE_p_OsiClpSolverInterface solver2 = javacbc.newSolver();

		//Load the problem into it
		javacbc.loadProblem(solver2, matrix, colLower, colUpper, objective, rowLower, rowUpper);

		//Make all variables integer
		for(int k=0;k<8;k++) javacbc.setInteger(solver2,k);

		//Write the associated lp file?
		//javacbc.writeLp(solver2, "myproblem");

		//One way to add a new constraint:
		//int column[]={0,1,2,3,4,5,6,7};
		//double element2[] ={1,1,1,1,1,1,1,1};
		//javacbc.addRow(solver2,8,column,element2,8,100); //Solver, number of non-zeros, indices of the columns, coefficients, lhs, rhs

		//Make a CbcModel
		SWIGTYPE_p_CbcModel model = javacbc.makeCbcModel(solver2);
		javacbc.setLogLevel(model,1);
		javacbc.setObjSense(model,-1.0); //Maximization
		javacbc.initialSolve(model);
		javacbc.branchAndBound(model,0);



		//Test and show result
		showResult(model);
		System.out.println("########### Test #5 ###########\n");

	}


	//Utilities: takes a two-dimensional dense array, and makes it sparse and flattened for use with CbcModel's methods
	public double[][] makeSparseAndFlatten(double[][] in){
		double[][] out = new double[3][];
		//Dims of in, supposes in[row][col]
		int nbcol = in[0].length;
		int nbrow = in.length;
		//count the number of non-zeros
		int nbNZ = countNZ(in);

		for (int i=0;i<nbrow;i++) out[i] = new double[nbNZ];

		int ind = 0;
		for (int i=0;i<nbrow;i++){
			for (int j=0;j<nbcol;j++){
				if (in[i][j]!=0.0){
					out[0][ind]=i; //Pos in row
					out[1][ind]=j; //Pos in col
					out[2][ind]=in[i][j]; //element value
					ind++;
				}
			}
		}

		return out;
	}

	//Count the number of non-zeros
	public int countNZ(double[][] in){
		int nbNZ=0;

		int nbcol = in[0].length;
		int nbrow = in.length;
		
		for (int i=0;i<nbrow;i++) for (int j=0;j<nbcol;j++) if (in[i][j]!=0.0) nbNZ++;

		return nbNZ;
	}

	//Test if a solution was found and display it
	public static void showResult(SWIGTYPE_p_CbcModel model){
		if (javacbc.isProvenOptimal(model)){
			System.out.println("Objective value:" + javacbc.getObjValue(model));
			System.out.println("Solution:" + Arrays.toString(javacbc.bestSolution(model)));
		}else{
			System.out.println("Best continuous value:" + javacbc.getCurrentObjValue(model));
			System.out.println("Infeasible!");
		}
	}
}
